# Juju charm for deploying content to Apache

## How to deploy the charm

Use this charm as a subordinate to a charm like apache2. It will download the
content and install it on the server.

## Supported sources

Currently swift, bzr+ssh, file, lp, http, https and ftp are supported. http,
https and ftp access can either be anonymous or via a username and
password. bzr+ssh requires a private key to be shipped with the charm, this
should be a private key which is used for nothing else. swift requires the
full gamut of authentication information.

dest_dir is the location to expand the archive into. In the case of apache
this would most likely be the document root specified in the vhost

keep_dir_count is the number of backups of the deployed content to keep.
Before each deploy the current content is archived before the new content is
pulled in.

deploy_trigger is used to trigger a deploy of new content. To trigger the
charm to pickup new content increment this. Note that setting any other
parameter will not trigger a deployment.

Examples (not exhaustive:)
http, https and ftp:
    archive_location=http://192.168.0.1/content.tar
    archive_location=https://user@pass@192.168.0.1:4343/content.tar
    archive_location=ftp://192.168.0.1/content.tar
    archive_location=ftp://user:pass@192.168.0.1:912/content.tar

swift:
    username=$OS_USERNAME
    secret=$OS_PASSWORD
    archive_location=content.tar
    auth_url=$OS_AUTH_URL
    tenant_name=$OS_TENANT_NAME
    container_name=my_container

bzr+ssh:
 Copy private key to ssh-key/content_fetcher_rsa within the charm
    archive_location=bzr+ssh://%s@%s/%s
    ssh_host_key=<host key of server with content>

file:
    archive_location=file:///srv/content.tar

lp:
    archive_location=lp:mybranch

### High level example:
Below is how the charms relate to one another
    juju deploy apache2
    juju deploy content-fetcher archive_location=http://192.168.0.1/content.tar
    juju add-relation apache2:juju-info content-fetcher:juju-info

## How to upgrade website content:
    juju set content-fetcher deploy_trigger=<current_value + 1>

## Testing charm on lxc:
For lxc deployments test-content-fetcher.py can be run directly on the host orchestrating the deployment

## Testing charm on other providers:
Deploy an instance to act as the deployment server in the cloud and copy test-content-fetcher.py onto it. i.e.
    juju deploy ubuntu
    juju scp precise/content-fetcher/tests/testserver ubuntu/0:~/ -r
    juju ssh ubuntu/0 "sudo ./tests/testserver/testserver-setup.sh"
    juju ssh ubuntu/0

Finally source your novarc file on the ubuntu/0 before invoking test-content-fetcher.py for swift test
Now run any test-content-fetcher.py commands below in the ubuntu/0 shell

### Get a local copy of the charm as it contains the test server
    bzr branch lp:charms/precise/content-fetcher
    
### Deploy Charms
    juju bootstrap
    juju deploy apache2
    juju set apache2 "servername=mywebsite.mydomain"
    juju deploy local:content-fetcher
    juju set content-fetcher "dest_dir=/srv/mywebsite.mydomain/mywebsite"
    juju add-relation apache2:juju-info content-fetcher:juju-info
    juju set apache2 "vhost_http_template=$(base64 testserver/apache-vhost.txt)"

### Start a test server.
    test-content-fetcher.py can be used to test all src types except lp. In the
    cases of http, https and ftp a server will be started for content-fetcher to 
    talk to, optionally --anon can be passed to turn of auth. For swift the
    content is uploaded into a new container. For bzr+ssh and file the archive
    is created. For all types instructions are then provided on how to setup the
    test and what juju params to set e.g.

    ./tests/testserver/test-content-fetcher.py https    

    This will start an https server and return instructions on what juju
    commands to run to instruct content-fetcher to grab its content from
    a basic auth protected https location. e.g.

    juju set content-fetcher archive_location=https://bob:bob@192.168.0.1:4343/content.tar
    juju set content-fetcher deploy_trigger=$(juju get content-fetcher | grep -A6 deploy_trigger | awk '/value:/ {print $2 + 1}')

    This can be tested with:
    juju ssh content-fetcher/0 "curl http://127.0.0.1/"

    This should return something like: <html><body><h1>Website collected from https auth at 2014/03/20 18:07:11</h1></body></html>

## Restrictions

## Todo
